@@ -2,52 +2,43 @@
 Core classes for validation.
 """
 
-from . import declarative
 import gettext
 import os
 import re
 import textwrap
 import warnings
 
-try:
-    from pkg_resources import resource_filename
-except ImportError:
-    resource_filename = None
+from . import declarative
 
 __all__ = ['NoDefault', 'Invalid', 'Validator', 'Identity',
            'FancyValidator', 'is_empty', 'is_validator']
 
 
 def get_localedir():
-    """Retrieve the location of locales.
-
-    If we're built as an egg, we need to find the resource within the egg.
-    Otherwise, we need to look for the locales on the filesystem or in the
-    system message catalog.
-
-    """
-    locale_dir = ''
-    # Check the egg first
-    if resource_filename is not None:
-        try:
-            locale_dir = resource_filename(__name__, "/i18n")
-        except NotImplementedError:
-            # resource_filename doesn't work with non-egg zip files
-            pass
+    """Retrieve the location of locales."""
+    file_dir = os.path.join(os.path.dirname(__file__), 'i18n')
     if not hasattr(os, 'access'):
-        # This happens on Google App Engine
-        return os.path.join(os.path.dirname(__file__), 'i18n')
-    if os.access(locale_dir, os.R_OK | os.X_OK):
-        # If the resource is present in the egg, use it
-        return locale_dir
+        # this happens on Google App Engine
+        return file_dir
 
-    # Otherwise, search the filesystem
-    locale_dir = os.path.join(os.path.dirname(__file__), 'i18n')
-    if not os.access(locale_dir, os.R_OK | os.X_OK):
-        # Fallback on the system catalog
-        locale_dir = os.path.normpath('/usr/share/locale')
+    import importlib.resources as resources
+    # try to get it via the resource
+    pkg_name = __name__.split('.', 1)[0]
+    try:
+        resource_dir = resources.files(pkg_name) / 'i18n'
+    except AttributeError:  # Python < 3.11
+        with resources.path(pkg_name, 'i18n') as resource_dir:
+            pass
+    if os.access(resource_dir, os.R_OK | os.X_OK):
+        # if the resource is present, use it
+        return resource_dir
 
-    return locale_dir
+    # otherwise, search the filesystem
+    if os.access(file_dir, os.R_OK | os.X_OK):
+        return file_dir
+
+    # fallback on the system catalog
+    return os.path.normpath('/usr/share/locale')
 
 
 def set_stdtranslation(domain="FormEncode", languages=None,
@@ -57,18 +48,17 @@ def set_stdtranslation(domain="FormEncode", languages=None,
                             languages=languages,
                             localedir=localedir, fallback=True)
     global _stdtrans
-    try:
-        _stdtrans = t.ugettext
-    except AttributeError:  # Python 3
-        _stdtrans = t.gettext
+    _stdtrans = t.gettext
+
 
 set_stdtranslation()
 
 # Dummy i18n translation function, nothing is translated here.
-# Instead this is actually done in api.Validator.message.
+# Instead, this is actually done in api.Validator.message.
 # The surrounding _('string') of the strings is only for extracting
 # the strings automatically.
-# If you run pygettext with this source comment this function out temporarily.
+# If you run pygettext with this source, comment this function out temporarily.
+
 _ = lambda s: s
 
 
@@ -90,13 +80,13 @@ def deprecated(old=None, new=None):
     return outer
 
 
-class NoDefault(object):
+class NoDefault:
     """A dummy value used for parameters with no default."""
 
 
 def is_empty(value):
     """Check whether the given value should be considered "empty"."""
-    return value is None or value == '' or (
+    return value is None or value == '' or value == b'' or (
         isinstance(value, (list, tuple, dict)) and not value)
 
 
@@ -147,18 +137,7 @@ class Invalid(Exception):
                 % (self, self.error_list, self.error_dict))
 
     def __str__(self):
-        val = self.msg
-        return val
-
-    if str is not str:  # Python 2
-
-        def __unicode__(self):
-            if isinstance(self.msg, str):
-                return self.msg
-            elif isinstance(self.msg, str):
-                return self.msg.decode('utf8')
-            else:
-                return str(self.msg)
+        return self.msg
 
     def unpack_errors(self, encode_variables=False, dict_char='.',
                       list_char='-'):
@@ -194,9 +173,7 @@ class Invalid(Exception):
         return self.msg
 
 
-############################################################
-## Base Classes
-############################################################
+# Base Classes
 
 class Validator(declarative.Declarative):
 
@@ -255,8 +232,7 @@ class Validator(declarative.Declarative):
         if not callable(trans):
             trans = _stdtrans
 
-        msg = self._messages[msgName]
-        msg = trans(msg, **self.gettextargs)
+        msg = trans(self._messages[msgName], **self.gettextargs)
 
         try:
             return msg % kw
@@ -309,11 +285,10 @@ class Validator(declarative.Declarative):
         This changes the class's docstring to include information
         about all the messages this validator uses.
         """
-        doc = cls.__doc__ or ''
-        doc = [textwrap.dedent(doc).rstrip()]
-        messages = sorted(cls._messages.items())
-        doc.append('\n\n**Messages**\n\n')
-        for name, default in messages:
+        doc = [
+            textwrap.dedent(cls.__doc__ or '').rstrip(),
+            '\n\n**Messages**\n\n']
+        for name, default in sorted(cls._messages.items()):
             default = re.sub(r'(%\(.*?\)[rsifcx])', r'``\1``', default)
             doc.append('``' + name + '``:\n')
             doc.append('  ' + default + '\n\n')
@@ -325,6 +300,7 @@ class _Identity(Validator):
     def __repr__(self):
         return 'validators.Identity'
 
+
 Identity = _Identity()
 
 
@@ -451,8 +427,7 @@ class FancyValidator(Validator):
                         stacklevel=cls._inheritance_level + 2)
                     setattr(cls, new, new_attrs[old])
             elif new in new_attrs:
-                    setattr(cls, old, deprecated(old=old, new=new)(
-                        new_attrs[new]))
+                setattr(cls, old, deprecated(old=old, new=new)(new_attrs[new]))
 
     def to_python(self, value, state=None):
         try:
